package com.book;

import java.beans.PropertyVetoException;

import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;

import org.axonframework.commandhandling.model.GenericJpaRepository;
import org.axonframework.commandhandling.model.Repository;
import org.axonframework.common.jpa.ContainerManagedEntityManagerProvider;
import org.axonframework.common.jpa.EntityManagerProvider;
import org.axonframework.eventhandling.EventBus;
import org.axonframework.eventsourcing.eventstore.EventStorageEngine;
import org.axonframework.eventsourcing.eventstore.inmemory.InMemoryEventStorageEngine;
import org.axonframework.spring.config.EnableAxonAutoConfiguration;
import org.axonframework.spring.config.TransactionManagerFactoryBean;
import org.axonframework.spring.messaging.unitofwork.SpringTransactionManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.transaction.PlatformTransactionManager;

import com.book.domain.Book;
import com.book.repository.entity.BookEntity;


@EnableAxonAutoConfiguration
@SpringBootApplication
public class BookCqrsApplication {

	public static void main(String[] args) {
		SpringApplication.run(BookCqrsApplication.class, args);
	}
	
	@Bean
	public EventStorageEngine eventStorageEngine() {
		return new InMemoryEventStorageEngine();
	}

	@Bean
	public Repository<Book> jpaRepo(EventBus eventBus) {
		return new GenericJpaRepository<>(entityManagerProvider(), Book.class, eventBus);
	}

	@Bean
	public EntityManagerProvider entityManagerProvider() {
		return new ContainerManagedEntityManagerProvider();
	}

	@PersistenceUnit
	private EntityManagerFactory entityManagerFactory;

	@Autowired
	private SpringTransactionManager springTransactionManager;

	@Bean
	public SpringTransactionManager springTransactionManager() throws Exception {
		return new SpringTransactionManager(transactionManager());
	}

	@Bean
	public PlatformTransactionManager transactionManager() throws PropertyVetoException {
		return new JpaTransactionManager(entityManagerFactory);
	}

	@Bean
	public TransactionManagerFactoryBean transactionManagerFactoryBean() throws PropertyVetoException {
		TransactionManagerFactoryBean factoryBean = new TransactionManagerFactoryBean();
		factoryBean.setTransactionManager(transactionManager());

		return factoryBean;
	}

}
