package com.book.domain;

import static org.axonframework.commandhandling.model.AggregateLifecycle.apply;

import org.axonframework.commandhandling.CommandHandler;
import org.axonframework.commandhandling.model.AggregateIdentifier;
import org.axonframework.eventhandling.EventHandler;
import org.axonframework.spring.stereotype.Aggregate;

import com.book.command.AddBookCommand;
import com.book.command.UpdateBookCommand;
import com.book.event.BookAddedEvent;
import com.book.event.BookUpdatedEvent;

@Aggregate
public class Book {
	@AggregateIdentifier
	private Long bookId;
	private String bookName;
	private int price;

	public Book() {

	}

	@CommandHandler
	public Book(AddBookCommand addBookCommand) {
		apply(new BookAddedEvent(addBookCommand.getBookId(), addBookCommand.getBookName(), addBookCommand.getPrice()));
	}

	@CommandHandler
	public void updateBookCommand(UpdateBookCommand updateBookCommand) {
		apply(new BookUpdatedEvent(updateBookCommand.getBookId(), updateBookCommand.getBookName(),
				updateBookCommand.getPrice()));
	}

	@EventHandler
	public void onBookAddedEvent(BookAddedEvent addedEvent) {
		this.bookId = addedEvent.getBookId();
		this.bookName = addedEvent.getBookName();
		this.price = addedEvent.getPrice();
	}

	@EventHandler
	public void onUpdateBookEvent(BookUpdatedEvent bookUpdatedEvent) {
		this.bookId = bookUpdatedEvent.getBookId();
		this.bookName = bookUpdatedEvent.getBookName();
		this.price = bookUpdatedEvent.getPrice();
	}

	public Long getBookId() {
		return bookId;
	}

	public void setBookId(Long bookId) {
		this.bookId = bookId;
	}

	public String getBookName() {
		return bookName;
	}

	public void setBookName(String bookName) {
		this.bookName = bookName;
	}

	public int getPrice() {
		return price;
	}

	public void setPrice(int price) {
		this.price = price;
	}
	
	

}
